# vim: set fileencoding=utf-8 :
#
# Copyright (C) 2010-2012 Marnix H. Medema
# University of Groningen
# Department of Microbial Physiology / Groningen Bioinformatics Centre
#
# Copyright (C) 2011,2012 Kai Blin
# University of Tuebingen
# Interfaculty Institute of Microbiology and Infection Medicine
# Div. of Microbiology/Biotechnology
#
# License: GNU Affero General Public License v3 or later
# A copy of GNU AGPL v3 should have been included in this software package in LICENSE.txt.

from pyquery import PyQuery as pq
from antismash import utils

def will_handle(product):
    if product.find('lantipeptide') > -1:
        return True

    return False

def generate_details_div(cluster, seq_record, options, js_domains, details=None):
    """Generate details div"""

    cluster_rec = utils.get_cluster_by_nr(seq_record, cluster['idx'])
    if cluster_rec is None:
        return details

    core_peptides = _find_core_peptides(cluster_rec, seq_record)

    if details is None:
        details = pq('<div>')
        details.addClass('details')

        header = pq('<h3>')
        header.text('Detailed annotation')
        details.append(header)

    if len(core_peptides) == 0:
        details_text = pq('<div>')
        details_text.addClass('details-text')
        details_text.text('No core peptides found.')
        details.append(details_text)
        return details

    details_text = pq('<dl>')
    details_text.addClass('details-text')

    for cp in core_peptides:
        core_seq = _get_core_peptide_sequence(cp)
        dt = pq('<dt>')
        dt.text('%s core peptide, putative %s' % (utils.get_gene_id(cp),
                _get_core_peptide_class(cp)))
        details_text.append(dt)

        dd = pq('<dd>')
        core_seq = core_seq.replace('S', '<span class="dha">Dha</span>')
        core_seq = core_seq.replace('T', '<span class="dhb">Dhb</span>')
        core_seq = core_seq.replace('C', '<span class="cys">C</span>')
        dd.html(core_seq)
        details_text.append(dd)

    details.append(details_text)

    legend = pq('<div>')
    legend.addClass('legend')
    legend_header = pq('<h4>')
    legend_header.text('Legend:')
    legend.append(legend_header)

    legend_text = pq('<div>')
    legend_text.html('<span class="dha">Dha</span>: Didehydroalanine<br>'
                '<span class="dhb">Dhb</span>: Didehydrobutyrine')
    legend.append(legend_text)
    details.append(legend)

    return details

def generate_sidepanel(cluster, seq_record, options, sidepanel=None):
    """Generate sidepanel div"""
    cluster_rec = utils.get_cluster_by_nr(seq_record, cluster['idx'])
    if cluster_rec is None:
        return sidepanel

    if sidepanel is None:
        sidepanel = pq('<div>')
        sidepanel.addClass('sidepanel')

    core_peptides = _find_core_peptides(cluster_rec, seq_record)
    if len(core_peptides) == 0:
        return sidepanel

    details = pq('<div>')
    details.addClass('more-details')
    details_header = pq('<h3>')
    details_header.text('Prediction details')
    details.append(details_header)
    details_list = pq('<dl>')
    details_list.addClass('prediction-text')

    for cp in core_peptides:
        dt = pq('<dt>')
        dt.text(utils.get_gene_id(cp))
        details_list.append(dt)
        dd = pq('<dd>')
        mol_weight = _get_molecular_weight(cp)
        bridges = _get_number_bridges(cp)
        pred_class = _get_core_peptide_class(cp)
        dd.html('Putative %s<br>Molecular weight: %s<br>Number of bridges: %s' %\
                (pred_class, mol_weight, bridges))
        details_list.append(dd)

    details.append(details_list)
    sidepanel.append(details)

    return sidepanel

def _find_core_peptides(cluster, seq_record):
    """Find CDS_motifs containing lantipeptide core peptide annotations"""
    motifs = []
    for motif in utils.get_all_features_of_type(seq_record, 'CDS_motif'):
        if motif.location.start < cluster.location.start or \
           motif.location.end > cluster.location.end:
            continue

        if not motif.qualifiers.has_key('note'):
            continue

        if not 'core peptide' in motif.qualifiers['note']:
            continue

        motifs.append(motif)

    return motifs

def _get_molecular_weight(motif):
    """Get molecular weight of a core peptide motif"""
    for note in motif.qualifiers['note']:
        if not note.startswith('molecular weight:'):
            continue
        return int(note.split(':')[-1])

def _get_number_bridges(motif):
    """Get number of bridges of a core peptide motif"""
    for note in motif.qualifiers['note']:
        if not note.startswith('number of bridges:'):
            continue
        return int(note.split(':')[-1])

def _get_core_peptide_sequence(motif):
    """Get AA sequence of a core peptide motif"""
    for note in motif.qualifiers['note']:
        if not note.startswith('predicted core seq:'):
            continue
        return note.split(':')[-1].strip()

def _get_core_peptide_class(motif):
    """Get the predicted class of the core peptide"""
    for note in motif.qualifiers['note']:
        if not note.startswith('predicted class:'):
            continue
        return note.split(':')[-1].strip().replace('-', ' ')
